#include <stdlib.h>
#include <stdio.h>

#include "itkImage.h"

#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"

#include "FilterProgressPrinter.h"

int main( int argc, char *argv[] )
{
  char * in_file_name  = argv[1];
  char * out_file_name = argv[2];
  const unsigned int divisions = atoi( argv[3] );

  const unsigned int Dimension = 3;
  typedef itk::Image< float, Dimension >           ScalarImageType;

  // Observer
  FilterProgressPrinter::Pointer observer = FilterProgressPrinter::New();

  // Reader
  typedef itk::ImageFileReader< ScalarImageType > FileReaderType;
  FileReaderType::Pointer reader = FileReaderType::New();
  reader->SetFileName( in_file_name );
  reader->AddObserver( itk::ProgressEvent(), observer );

  // Writer
  typedef itk::ImageFileWriter< ScalarImageType > FileWriterType;
  FileWriterType::Pointer writer = FileWriterType::New();
  writer->SetFileName( out_file_name );
  writer->SetInput( reader->GetOutput() );
  writer->SetNumberOfStreamDivisions( divisions );
  writer->AddObserver( itk::ProgressEvent(), observer );

  try 
    {
    writer->Update();
    }
  catch (itk::ExceptionObject &e)
    {
    std::cerr << e << std::endl;
    }


  return EXIT_SUCCESS;
}
