#include <stdlib.h>
#include <stdio.h>

#include "itkRGBPixel.h"
#include "itkVector.h"
#include "itkImage.h"

#include "itkImageFileReader.h"
#include "itkVectorCastImageFilter.h"
#include "itkVectorGradientMagnitudeImageFilter.h"
#include "itkImageFileWriter.h"

#include "FilterProgressPrinter.h"

int main( int argc, char *argv[] )
{
  char * in_file_name  = argv[1];
  char * out_file_name = argv[2];
  const unsigned int divisions = atoi( argv[3] );

  const unsigned int Dimension = 3;
  const unsigned int Channels = 3;
  typedef itk::RGBPixel <unsigned char >           RGBPixelType;
  typedef itk::Image< RGBPixelType, Dimension >    RGBImageType;
  typedef itk::Vector< float, Channels >           VectorPixelType;
  typedef itk::Image< VectorPixelType, Dimension > VectorImageType;
  typedef itk::Image< float, Dimension >           ScalarImageType;

  // Observer
  FilterProgressPrinter::Pointer observer = FilterProgressPrinter::New();

  // Reader
  typedef itk::ImageFileReader< RGBImageType > FileReaderType;
  FileReaderType::Pointer reader = FileReaderType::New();
  reader->SetFileName( in_file_name );
  reader->AddObserver( itk::ProgressEvent(), observer );

  // Cast Filter
  typedef itk::VectorCastImageFilter
    < RGBImageType, VectorImageType > CastFilterType;
  CastFilterType::Pointer caster = CastFilterType::New();
  caster->SetInput( reader->GetOutput() );
  caster->AddObserver( itk::ProgressEvent(), observer );

  // Gradient Magnitude Filter
  typedef itk::VectorGradientMagnitudeImageFilter
    < VectorImageType > GradientMagnitudeFilterType;
  GradientMagnitudeFilterType::Pointer 
    gradient = GradientMagnitudeFilterType::New();
  gradient->SetInput( caster->GetOutput() );
  gradient->AddObserver( itk::ProgressEvent(), observer );

  // Writer
  typedef itk::ImageFileWriter< ScalarImageType > FileWriterType;
  FileWriterType::Pointer writer = FileWriterType::New();
  writer->SetFileName( out_file_name );
  writer->SetInput( gradient->GetOutput() );
  writer->SetNumberOfStreamDivisions( divisions );
  writer->AddObserver( itk::ProgressEvent(), observer );

  try 
    {
    writer->Update();
    }
  catch (itk::ExceptionObject &e)
    {
    std::cerr << e << std::endl;
    }


  return EXIT_SUCCESS;
}
